/* 
 * Copyright (c) 2007 Anthony Mallet                  --  Fri Jan 19 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <stdio.h>

#include "viam_private.h"

#ifdef VIAM_OPENCV_SUPPORT

#ifndef CV_INLINE
# define CV_INLINE	static inline
#endif
#define CV_NO_BACKWARD_COMPATIBILITY
#include <cxtypes.h>
#include <cv.h>


/* --- viam_awb_auto ------------------------------------------------- */

/** \brief White-balance correction
 *
 * Apply U (blue) and V (red) gain.
 * 
 * \param[in] vh	Library handle.
 * \param[in] source	BGR image on which to perform analysis.
 * \param[out] result	Computed image.
 * \param[inout] args	Gains
 *
 * \return 0 on sucess.
 */


int
viam_awb(viam_handle_t vh, viam_image_t image, const IplImage *source,
	 IplImage *result, viam_filterargs_t *args, void *data)
{
  int i, j;
  CvScalar p;

  /* compute B-G and R-G averages */
  for(i=0; i<source->height; i++)
    for(j=0; j<source->width; j++) {
      p = cvGet2D(source, i, j);
      p.val[0] *= args->blue;
      /*       p.val[1]; */
      p.val[2] *= args->red;
      cvSet2D(result, i, j, p);
    }

  printf("%g %g\n", args->blue, args->red);
  return 0;
}


/* --- viam_awb_auto ------------------------------------------------- */

/** \brief Auto white-balance algorithm
 *
 * Compute U (blue) and V (red) gain so that red, green and blue
 * pixel average are equal.
 * 
 * \param[in] src	BGR image on which to perform analysis.
 * \param[out] u	Computed blue gain (U color component).
 * \param[out] v	Computed red gain (V color component).
 *
 * \return 0 on sucess.
 */

int
viam_awb_auto(viam_handle_t vh, viam_image_t image, const IplImage *source,
	      viam_filterargs_t *args, void *data)
{
  int i, j;
  CvScalar p;
  double avb, avg, avr;

  /* compute B-G and R-G averages */
  avb = avr = 0.;
  for(i=0; i<source->height; i++)
    for(j=0; j<source->width; j++) {
      p = cvGet2D(source, i, j);

      avb += p.val[0];
      avg += p.val[1];
      avr += p.val[2];
    }

  args->blue = avg/avb;
  args->red = avg/avr;
  return 0;
}


#endif /* VIAM_OPENCV_SUPPORT */
