/*
 * Copyright (c) 2007-2008
 *     IS/AIST-ST2I/CNRS Joint Japanese-French Robotics Laboratory (JRL).
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <err.h>

#include "viam_private.h"

#ifdef VIAM_DC1394_SUPPORT

#include <dc1394/control.h>


/* --- local data ---------------------------------------------------- */

typedef struct viam1394_camera_t {
  dc1394camera_t *handle;		/**< libdc1394 camera handle. */
  dc1394featureset_t featset;		/**< camera features. */
  dc1394video_frame_t *frame;		/**< last captured frame. */
} *viam1394_camera_t;

static dc1394_t *dc1394h;		/**< general libdc134 handle */

static void	viam1394_camerainfo(dc1394camera_t *camera);
static int	viam1394_fwfeature(viam_filtertype_t filter,
			unsigned int *fwfeat);

static int	viam1394_hw_geo(const viam_hwmode_t *swgeo,
			dc1394video_mode_t *hwgeo);
static int	viam1394_sw_geo(dc1394video_mode_t hwgeo, viam_hwmode_t *swgeo);
static int	viam1394_hw_fps(viam_hwfps_t swfps, dc1394framerate_t *hwfps);
static int	viam1394_sw_fps(dc1394framerate_t hwfps, viam_hwfps_t *swfps);


/* --- viam1394_init ------------------------------------------------- */

int
viam1394_init(viam_handle_t vh)
{
  dc1394h = dc1394_new();
  if (!dc1394h) return viam_seterrno(vh, Eviamlib_FWBUS);

  return 0;
}


/* --- viam1394_fini ------------------------------------------------- */

int
viam1394_fini(viam_handle_t vh)
{
  if (dc1394h) dc1394_free(dc1394h);

  return 0;
}


/* --- viam1394_hardware_list ---------------------------------------- */

/** \brief Print available camera devices on all buses
 *
 * \param[in] vh	Library handle.
 *
 * \return 0 on success.
 */

int
viam1394_hardware_list(viam_handle_t vh)
{
  dc1394camera_list_t *clist;
  dc1394camera_t *camera;
  dc1394error_t s;
  int i;

  s = dc1394_camera_enumerate(dc1394h, &clist);
  if (s != DC1394_SUCCESS)
    return viam_seterrno(vh, Eviamlib_NO_CAMERA);

  /* iterate over all cameras */
  for(i=0; i<clist->num; i++) {
    camera = dc1394_camera_new_unit(
      dc1394h, clist->ids[i].guid, clist->ids[i].unit);
    if (!camera) continue;

    viam1394_camerainfo(camera);
    dc1394_camera_free(camera);
  }
  dc1394_camera_free_list(clist);

  return 0;
}


/* --- viam1394_hardware_attach -------------------------------------- */

/** \brief Allocate camera ports and handles
 *
 * \param[in] vh	Library handle.
 * \param[in] camera	Camera handle to attach.
 *
 * \return 0 on success, or an error.
 */

int
viam1394_hardware_attach(viam_handle_t vh, viam_camera_t camera)
{
  viam1394_camera_t c;
  uint64_t uid;
  char *end;
  int i;

  dc1394camera_list_t *clist;
  dc1394error_t s;

  /* parse uid into a 64bits integer. XXX for now assume no unit, but
   * unit support should be added at this point with a camera id like
   * dc1394:0xdeadbeef:n with n the unit number. */
  uid = strtoll(viam_camera_uid(camera), &end, 0);
  if (end == viam_camera_uid(camera) || *end != '\0')
    return viam_seterrno(vh, Eviamlib_BAD_ID);

  /* get all cameras */
  s = dc1394_camera_enumerate(dc1394h, &clist);
  if (s != DC1394_SUCCESS)
    return viam_seterrno(vh, Eviamlib_NO_CAMERA);

  /* scan all cameras and assign handle if one camera matches */
  for(i=0, c=NULL; i<clist->num; i++) {
    if (clist->ids[i].guid != uid) continue;

    /* register camera */
    c = malloc(sizeof(*c));
    if (!c) {
      warnx("unable to allocate memory for camera %d", i);
      break;
    }
    c->handle = dc1394_camera_new_unit(
      dc1394h, clist->ids[i].guid, clist->ids[i].unit);
    if (!c->handle) {
      warnx("unable to attach to camera `%s'", viam_camera_name(camera));
      free(c);
      c = NULL;
      break;
    }

    s = dc1394_feature_get_all(c->handle, &c->featset);
    if (s != DC1394_SUCCESS) {
      warnx("unable to get camera `%s' features", viam_camera_name(camera));
      dc1394_camera_free(c->handle);
      free(c);
      c = NULL;
      break;
    }

    s = dc1394_video_set_transmission(c->handle, DC1394_OFF);
    if (s != DC1394_SUCCESS) viam_seterrno(vh, Eviamlib_FWBUS);

    viam_camera_sethwdata(camera, c);

    warnx("camera `%s' is %s, %s",
	  viam_camera_name(camera), c->handle->model, c->handle->vendor);
    break;
  }

  dc1394_camera_free_list(clist);

  if (c == NULL) return viam_seterrno(vh, Eviamlib_NO_SUCH_CAMERA);
  return 0;
}


/* --- viam1394_hardware_detach -------------------------------------- */

/** \brief Cleanup dc1394 handle
 *
 * \param[in] vh	Library handle.
 * \param[in] camera	Camera handle.
 */

void
viam1394_hardware_detach(viam_handle_t vh, viam_camera_t camera)
{
  viam1394_camera_t hw;

  hw = viam_camera_hwdata(camera);
  if (hw) {
    viam_camera_setsetup(camera, 0);
    viam_camera_sethwdata(camera, NULL);
    dc1394_camera_free(hw->handle);
    free(hw);
  }
}


/* --- viam1394_camera_configure ------------------------------------- */

/** \brief Setup camera with current configuration parameters.
 *
 * \param[in] vh	Library handle.
 * \param[in] camera	Camera handle.
 *
 * \return 0 on success
 */

int
viam1394_camera_configure(viam_handle_t vh, viam_camera_t camera)
{
  viam1394_camera_t c;
  dc1394video_mode_t fwmode;
  dc1394framerate_t fwfps;
  dc1394error_t s;

  c = viam_camera_hwdata(camera);
  if (!c) return viam_seterrno(vh, Eviamlib_NO_SUCH_CAMERA);

  /* configure hardware mode */
  if (viam1394_hw_geo(viam_camera_hwmode(vh, camera), &fwmode))
    return viam_seterrno(vh, Eviamlib_UNSUPPORTED_HWMODE);
  if (viam1394_hw_fps(viam_camera_hwmode(vh, camera)->fps, &fwfps))
    return viam_seterrno(vh, Eviamlib_UNSUPPORTED_HWMODE);

  s = dc1394_video_set_mode(c->handle, fwmode);
  if (s != DC1394_SUCCESS) return viam_seterrno(vh, Eviamlib_FWBUS);

  s = dc1394_video_set_framerate(c->handle, fwfps);
  if (s != DC1394_SUCCESS) return viam_seterrno(vh, Eviamlib_FWBUS);

  viam_camera_setsetup(camera, 1);
  s = dc1394_capture_setup(c->handle, 2/*dma buffers*/,
			   DC1394_CAPTURE_FLAGS_DEFAULT);
  if (s != DC1394_SUCCESS) {
    viam_camera_setsetup(camera, 0);
    return viam_seterrno(vh, Eviamlib_FWBUS);
  }

  return 0;
}


/* --- viam1394_camera_unconfigure ----------------------------------- */

/** \brief Reset bank configuration
 *
 * \param[in] vh	Library handle.
 * \param[in] camera	Camera handle.
 *
 * \return 0 on success
 */

int
viam1394_camera_unconfigure(viam_handle_t vh, viam_camera_t camera)
{
  viam1394_camera_t c;
  dc1394error_t s;

  c = viam_camera_hwdata(camera);
  if (!c) return 0;

  viam_camera_setsetup(camera, 0);
  s = dc1394_capture_stop(c->handle);
  if (s != DC1394_SUCCESS) {
    warnx("couldn't detach capture info from camera `%s'",
	  viam_camera_name(camera));
  }

  return 0;
}


/* --- viam_datatransmit --------------------------------------------- */

int
viam1394_datatransmit(viam_handle_t vh, viam_camera_t camera, viam_onoff_t on)
{
  viam1394_camera_t c;
  dc1394error_t s;

  /* switch off */
  c = viam_camera_hwdata(camera);
  if (!c) return viam_seterrno(vh, Eviamlib_NOT_CONFIGURED);

  s = dc1394_video_set_transmission(c->handle, DC1394_OFF);
  if (s != DC1394_SUCCESS) return viam_seterrno(vh, Eviamlib_FWBUS);

  if (!on) return 0;

  /* flush DMA buffer to skip old data */
  while(1) {
    s = dc1394_capture_dequeue(c->handle,
			       DC1394_CAPTURE_POLICY_POLL, &c->frame);
    if (s != DC1394_SUCCESS) return viam_seterrno(vh, Eviamlib_FWBUS);
    if (!c->frame) /* no more old frames */ break;

    s = dc1394_capture_enqueue(c->handle, c->frame);
    if (s != DC1394_SUCCESS) return viam_seterrno(vh, Eviamlib_FWBUS);
  }

  /* switch on */
  s = dc1394_video_set_transmission(c->handle, DC1394_ON);
  if (s != DC1394_SUCCESS) return viam_seterrno(vh, Eviamlib_FWBUS);

  return 0;
}


/* --- viam1394_capture ---------------------------------------------- */

int
viam1394_capture(viam_handle_t vh, viam_camera_t camera)
{
  viam1394_camera_t c;
  dc1394error_t s;

  c = viam_camera_hwdata(camera);
  if (!c) return viam_seterrno(vh, Eviamlib_NOT_CONFIGURED);

  s = dc1394_capture_dequeue(c->handle,
			     DC1394_CAPTURE_POLICY_WAIT, &c->frame);
  if (s != DC1394_SUCCESS) return viam_seterrno(vh, Eviamlib_FWBUS);

  return 0;
}


/* --- viam1394_capturedata ------------------------------------------ */

char *
viam1394_capturedata(viam_handle_t vh, viam_camera_t camera)
{
  viam1394_camera_t c = viam_camera_hwdata(camera);

  if (!c) { viam_seterrno(vh, Eviamlib_NOT_CONFIGURED); return NULL; }

  return (char *)c->frame->image;
}


/* --- viam1394_capturetimestamp ------------------------------------- */

struct timeval
viam1394_capturetimestamp(viam_handle_t vh, viam_camera_t camera)
{
  viam1394_camera_t c = viam_camera_hwdata(camera);
  struct timeval t = {0, 0};

  if (!c) { viam_seterrno(vh, Eviamlib_NOT_CONFIGURED); return t; }

  t.tv_sec = c->frame->timestamp / 1000000;
  t.tv_usec = c->frame->timestamp % 1000000;
  return t;
}


/* --- viam1394_capturedone ------------------------------------------ */

int
viam1394_capturedone(viam_handle_t vh, viam_camera_t camera)
{
  viam1394_camera_t c = viam_camera_hwdata(camera);
  dc1394error_t s;

  if (!c) return viam_seterrno(vh, Eviamlib_NOT_CONFIGURED);

  s = dc1394_capture_enqueue(c->handle, c->frame);
  if (s != DC1394_SUCCESS) return viam_seterrno(vh, Eviamlib_FWBUS);

  return 0;
}


/* --- viam1394_queryhwmodes ----------------------------------------- */

int
viam1394_queryhwmodes(viam_handle_t vh, viam_camera_t camera,
		      viam_hwmode_t *supported, size_t n)
{
  viam1394_camera_t c = viam_camera_hwdata(camera);
  dc1394video_modes_t modes;
  dc1394framerates_t framerates;
  viam_hwmode_t swmode;
  dc1394error_t s;
  int i, j, count;

  if (!c) { viam_seterrno(vh, Eviamlib_NOT_CONFIGURED); return 0; }

  s = dc1394_video_get_supported_modes(c->handle, &modes);
  if (s != DC1394_SUCCESS) { viam_seterrno(vh, Eviamlib_FWBUS); return 0; }

  count = 0;
  for(i=0; i<modes.num && count<n; i++) {
    if (viam1394_sw_geo(modes.modes[i], &swmode)) continue;

    s = dc1394_video_get_supported_framerates(
      c->handle, modes.modes[i], &framerates);
    if (s != DC1394_SUCCESS) {
      supported[count] = swmode;
      supported[count].fps = VIAM_HWFPS_INVALID;
      count++;
      continue;
    }

    for(j=0; j<framerates.num && count<n; j++) {
      supported[count] = swmode;
      if (!viam1394_sw_fps(framerates.framerates[j], &supported[count].fps))
	count++;
    }
  }
  return count;
}


/* --- viam1394_queryhwfeature --------------------------------------- */

int
viam1394_queryhwfeature(viam_handle_t vh,
			viam_camera_t camera, viam_filtertype_t filter,
			unsigned int *methods, unsigned int *aumodes)
{
  viam1394_camera_t c = viam_camera_hwdata(camera);
  dc1394feature_info_t *info;
  unsigned int feature;
  int i, s;

  if (methods) *methods = 0;
  if (aumodes) *aumodes = 0;

  if (!c) return viam_seterrno(vh, Eviamlib_NOT_CONFIGURED);

  s = viam1394_fwfeature(filter, &feature);
  if (s) return 0;

  info = &c->featset.feature[feature - DC1394_FEATURE_MIN];
  if (!info->available) return 0;
  if (info->on_off_capable && methods)
    *methods |= VIAM_FILTER_OFF;

  for(i=0;i<info->modes.num;i++) switch (info->modes.modes[i]) {
    case DC1394_FEATURE_MODE_MANUAL:
      if (methods) *methods |= VIAM_FILTER_HARDWARE;
      if (aumodes) *aumodes |= VIAM_FILTER_MANUAL;
      break;

    case DC1394_FEATURE_MODE_AUTO:
      if (aumodes) *aumodes |= VIAM_FILTER_HARDWARE_AUTO;
      break;

    case DC1394_FEATURE_MODE_ONE_PUSH_AUTO:
      if (aumodes) *aumodes |= VIAM_FILTER_HARDWARE_ONE_PUSH;
      break;
  }

  return 0;
}


/* --- viam1394_sethwfeature ----------------------------------------- */

/** \brief Set harware feature capabilities of a camera.
 *
 * \param[in] vh	Library handle.
 * \param[in] camera	Camera handle.
 * \param[in] filter	Feature type.
 * \param[in] method	Hardware mode.
 * \param[in] aumode	Auto mode.
 * \param[in] args	Feature value (NULL for AUTO or ONE_PUSH).
 *
 * \return 0 on success.
 */

int
viam1394_sethwfeature(viam_handle_t vh, viam_camera_t camera,
		      viam_filtertype_t filter, viam_filtermethod_t method,
		      viam_filterautomode_t aumode, const viam_filterargs_t *args)
{
  viam1394_camera_t c = viam_camera_hwdata(camera);
  unsigned int feature;
  dc1394feature_info_t *info;
  dc1394error_t s;

  if (!c) return viam_seterrno(vh, Eviamlib_NOT_CONFIGURED);

  s = viam1394_fwfeature(filter, &feature);
  if (s) return viam_seterrno(vh, s);
  info = &c->featset.feature[feature - DC1394_FEATURE_MIN];

  switch(method) {
    case VIAM_FILTER_OFF:
      s = dc1394_feature_set_power(c->handle, feature, DC1394_OFF);
      break;

    case VIAM_FILTER_HARDWARE:
      switch(aumode) {
	case VIAM_FILTER_MANUAL:
	case VIAM_FILTER_SOFTWARE_AUTO:
	case VIAM_FILTER_SOFTWARE_ONE_PUSH:
	  switch(feature) {
	    case DC1394_FEATURE_WHITE_BALANCE: {
	      unsigned int pb,pr, b,r;

	      s = dc1394_feature_whitebalance_get_value(c->handle, &pb, &pr);
	      if (s != DC1394_SUCCESS) return viam_seterrno(vh, Eviamlib_FWBUS);

	      if (args->blue > 0.0 && args->red > 0.0) {
		b = pb*0.75 + 0.25*pb*args->blue + 0.5;
		r = pr*0.75 + 0.25*pr*args->red + 0.5;
	      } else {
		b = r = (info->max + info->min)/2;
	      }

	      if (b > info->max-1) b = info->max -1;
	      if (b < info->min+1) b = info->min +1;
	      if (r > info->max-1) r = info->max -1;
	      if (r < info->min+1) r = info->min +1;

	      if (b != pb || r != pr) {
		printf("wb: %d %d | %d %d | %g %g\n",
		       pb, pr, b, r, args->blue, args->red);
		s = dc1394_feature_whitebalance_set_value(c->handle, b, r);
	      }
	      break;
	    }

	    default:
	      s = dc1394_feature_set_value(
		c->handle, feature,
		args->value*info->max + (1.0 - args->value)*info->min);
	      break;
	  }
	  break;

	case VIAM_FILTER_HARDWARE_AUTO:
	  s = dc1394_feature_set_mode(c->handle, feature, DC1394_FEATURE_MODE_AUTO);
	  break;

	case VIAM_FILTER_HARDWARE_ONE_PUSH:
	  s = dc1394_feature_set_mode(c->handle, feature,
				      DC1394_FEATURE_MODE_ONE_PUSH_AUTO);
	  break;

	case VIAM_FILTER_AUTOINVALID:
	  s = DC1394_FAILURE;
	  break;
      }
      break;

    default: return viam_seterrno(vh, Eviamlib_NOT_IMPLEMENTED);
  }
  if (s != DC1394_SUCCESS) return viam_seterrno(vh, Eviamlib_FWBUS);

  return 0;
}


/* --- viam1394_camerainfo ------------------------------------------- */

static void
viam1394_camerainfo(dc1394camera_t *camera)
{
  printf("[node 0x%016llx:%d] %s (%s)\n",
	 camera->guid, camera->unit, camera->model, camera->vendor);
}


/* --- viam1394_fwfeature -------------------------------------------- */

static const struct {
  unsigned int fwfeat;
} featuremap[] = {
  [VIAM_FILTER_NONE] =		{  -1 },
  [VIAM_FILTER_FORMAT] =	{  -1 },
  [VIAM_FILTER_SUBSAMPLE] =	{  -1 },
  [VIAM_FILTER_SCALE] =		{  -1 },
  [VIAM_FILTER_BRIGHTNESS] =	{ DC1394_FEATURE_BRIGHTNESS },
  [VIAM_FILTER_EXPOSURE] =	{ DC1394_FEATURE_EXPOSURE },
  [VIAM_FILTER_SHARPNESS] =	{ DC1394_FEATURE_SHARPNESS },
  [VIAM_FILTER_GAIN] =		{ DC1394_FEATURE_GAIN },
  [VIAM_FILTER_HUE] =		{ DC1394_FEATURE_HUE },
  [VIAM_FILTER_SATURATION] =	{ DC1394_FEATURE_SATURATION },
  [VIAM_FILTER_GAMMA] =		{ DC1394_FEATURE_GAMMA },
  [VIAM_FILTER_TEMPERATURE] =	{ DC1394_FEATURE_TEMPERATURE },
#ifdef VIAM_OPENCV_SUPPORT
  [VIAM_FILTER_WHITE_BALANCE] =	{ DC1394_FEATURE_WHITE_BALANCE },
  [VIAM_FILTER_BAYER_BGGR] =	{  -1 },
  [VIAM_FILTER_BAYER_GRBG] =	{  -1 },
  [VIAM_FILTER_BAYER_RGGB] =	{  -1 },
  [VIAM_FILTER_BAYER_GBRG] =	{  -1 },
  [VIAM_FILTER_RECTIFY] =	{  -1 }
#endif
};

static int
viam1394_fwfeature(viam_filtertype_t filter, unsigned int *fwfeat)
{
  *fwfeat = featuremap[filter].fwfeat;
  return *fwfeat == -1 ? Eviamlib_NOT_IMPLEMENTED : 0;
}


/* --- viam1394_geo -------------------------------------------------- */

static const struct {
  viam_hwmode_t sw;
  dc1394video_mode_t hw;
} geomap[] = {
  { { VIAM_HWSZ_160x120, VIAM_HWFMT_YUV444, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_160x120_YUV444 },

  { { VIAM_HWSZ_320x240, VIAM_HWFMT_YUV422, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_320x240_YUV422 },

  { { VIAM_HWSZ_640x480, VIAM_HWFMT_YUV411, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_640x480_YUV411 },

  { { VIAM_HWSZ_640x480, VIAM_HWFMT_YUV422, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_640x480_YUV422 },

  { { VIAM_HWSZ_640x480, VIAM_HWFMT_RGB888, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_640x480_RGB8 },

  { { VIAM_HWSZ_640x480, VIAM_HWFMT_MONO8,  VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_640x480_MONO8 },

  { { VIAM_HWSZ_640x480, VIAM_HWFMT_MONO16, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_640x480_MONO16 },

  { { VIAM_HWSZ_800x600, VIAM_HWFMT_YUV422, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_800x600_YUV422 },

  { { VIAM_HWSZ_800x600,   VIAM_HWFMT_RGB888, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_800x600_RGB8 },

  { { VIAM_HWSZ_800x600,   VIAM_HWFMT_MONO8,  VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_800x600_MONO8 },

  { { VIAM_HWSZ_1024x768,  VIAM_HWFMT_YUV422, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_1024x768_YUV422 },

  { { VIAM_HWSZ_1024x768,  VIAM_HWFMT_RGB888, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_1024x768_RGB8 },

  { { VIAM_HWSZ_1024x768,  VIAM_HWFMT_MONO8,  VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_1024x768_MONO8 },

  { { VIAM_HWSZ_800x600,   VIAM_HWFMT_MONO16, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_800x600_MONO16 },

  { { VIAM_HWSZ_1024x768,  VIAM_HWFMT_MONO16, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_1024x768_MONO16 },

  { { VIAM_HWSZ_1280x960,  VIAM_HWFMT_YUV422, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_1280x960_YUV422 },

  { { VIAM_HWSZ_1280x960,  VIAM_HWFMT_RGB888, VIAM_HW_FIXED } ,
    DC1394_VIDEO_MODE_1280x960_RGB8 },

  { { VIAM_HWSZ_1280x960,  VIAM_HWFMT_MONO8,  VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_1280x960_MONO8 },

  { { VIAM_HWSZ_1600x1200, VIAM_HWFMT_YUV422, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_1600x1200_YUV422 },

  { { VIAM_HWSZ_1600x1200, VIAM_HWFMT_RGB888, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_1600x1200_RGB8 },

  { { VIAM_HWSZ_1600x1200, VIAM_HWFMT_MONO8,  VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_1600x1200_MONO8 },

  { { VIAM_HWSZ_1280x960,  VIAM_HWFMT_MONO16, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_1280x960_MONO16 },

  { { VIAM_HWSZ_1600x1200, VIAM_HWFMT_MONO16, VIAM_HW_FIXED },
    DC1394_VIDEO_MODE_1600x1200_MONO16 }
};

static int
viam1394_hw_geo(const viam_hwmode_t *swgeo, dc1394video_mode_t *hwgeo)
{
  int i;
  if (!swgeo) return Eviamlib_UNSUPPORTED_HWMODE;

  for(i=0; i<sizeof(geomap)/sizeof(geomap[0]); i++)
    if (geomap[i].sw.size == swgeo->size &&
	geomap[i].sw.format == swgeo->format &&
	geomap[i].sw.crop == swgeo->crop) {
      *hwgeo = geomap[i].hw;
      return 0;
    }
  return Eviamlib_UNSUPPORTED_HWMODE;
}

static int
viam1394_sw_geo(dc1394video_mode_t hwgeo, viam_hwmode_t *swgeo)
{
  int i;

  for(i=0; i<sizeof(geomap)/sizeof(geomap[0]); i++)
    if (geomap[i].hw == hwgeo) {
      swgeo->size = geomap[i].sw.size;
      swgeo->format = geomap[i].sw.format;
      swgeo->crop = geomap[i].sw.crop;
      return 0;
    }
  return Eviamlib_UNSUPPORTED_HWMODE;
}


/* --- viam1394_fps -------------------------------------------------- */

static const struct {
  viam_hwfps_t sw;
  dc1394framerate_t hw;
} fpsmap[] = {
  { VIAM_HWFPS_1_875,	DC1394_FRAMERATE_1_875},
  { VIAM_HWFPS_3_75,	DC1394_FRAMERATE_3_75},
  { VIAM_HWFPS_7_5,	DC1394_FRAMERATE_7_5},
  { VIAM_HWFPS_15,	DC1394_FRAMERATE_15},
  { VIAM_HWFPS_30,	DC1394_FRAMERATE_30},
  { VIAM_HWFPS_60,	DC1394_FRAMERATE_60},
  { VIAM_HWFPS_120,	DC1394_FRAMERATE_120},
  { VIAM_HWFPS_240,	DC1394_FRAMERATE_240},
};

static int
viam1394_hw_fps(viam_hwfps_t swfps, dc1394framerate_t *hwfps)
{
  int i;

  for(i=0; i<sizeof(fpsmap)/sizeof(fpsmap[0]); i++)
    if (fpsmap[i].sw == swfps) {
      *hwfps = fpsmap[i].hw;
      return 0;
    }
  return Eviamlib_UNSUPPORTED_HWMODE;
}

static int
viam1394_sw_fps(dc1394framerate_t hwfps, viam_hwfps_t *swfps)
{
  int i;

  for(i=0; i<sizeof(fpsmap)/sizeof(fpsmap[0]); i++)
    if (fpsmap[i].hw == hwfps) {
      *swfps = fpsmap[i].sw;
      return 0;
    }
  return Eviamlib_UNSUPPORTED_HWMODE;
}

#endif /* VIAM_DC1394_SUPPORT */
